@@ -171,6 +171,131 @@ TREELITE_DLL int TreeliteLoadXGBoostJSONString(const char* json_str,
 TREELITE_DLL int TreeliteLoadXGBoostModelFromMemoryBuffer(const void* buf,
                                                           size_t len,
                                                           ModelHandle* out);
+
+/*!
+ * \brief Load a scikit-learn random forest regressor model from a collection of arrays. Refer to
+ *        https://scikit-learn.org/stable/auto_examples/tree/plot_unveil_tree_structure.html to
+ *        learn the mearning of the arrays in detail. Note that this function can also be used to
+ *        load an ensemble of extremely randomized trees (sklearn.ensemble.ExtraTreesRegressor).
+ * \param n_estimators number of trees in the random forest
+ * \param n_features number of features in the training data
+ * \param node_count node_count[i] stores the number of nodes in the i-th tree
+ * \param children_left children_left[i][k] stores the ID of the left child node of node k of the
+ *                      i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param children_right children_right[i][k] stores the ID of the right child node of node k of the
+ *                       i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param feature feature[i][k] stores the ID of the feature used in the binary tree split at node k
+ *                of the i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param threshold threshold[i][k] stores the threshold used in the binary tree split at node k of
+ *                  the i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param value value[i][k] stores the leaf output of node k of the i-th tree. This is only defined
+ *              if node k is a leaf node.
+ * \param n_node_samples n_node_samples[i][k] stores the number of data samples associated with
+ *                       node k of the i-th tree.
+ * \param impurity impurity[i][k] stores the impurity measure (gini, entropy etc) associated with
+ *                 node k of the i-th tree.
+ * \param out pointer to store the loaded model
+ * \return 0 for success, -1 for failure
+ */
+TREELITE_DLL int TreeliteLoadSKLearnRandomForestRegressor(
+    int n_estimators, int n_features, const int64_t* node_count, const int64_t** children_left,
+    const int64_t** children_right, const int64_t** feature, const double** threshold,
+    const double** value, const int64_t** n_node_samples, const double** impurity,
+    ModelHandle* out);
+
+/*!
+ * \brief Load a scikit-learn random forest classifier model from a collection of arrays. Refer to
+ *        https://scikit-learn.org/stable/auto_examples/tree/plot_unveil_tree_structure.html to
+ *        learn the mearning of the arrays in detail. Note that this function can also be used to
+ *        load an ensemble of extremely randomized trees (sklearn.ensemble.ExtraTreesClassifier).
+ * \param n_estimators number of trees in the random forest
+ * \param n_features number of features in the training data
+ * \param n_classes number of classes in the target variable
+ * \param node_count node_count[i] stores the number of nodes in the i-th tree
+ * \param children_left children_left[i][k] stores the ID of the left child node of node k of the
+ *                      i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param children_right children_right[i][k] stores the ID of the right child node of node k of the
+ *                       i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param feature feature[i][k] stores the ID of the feature used in the binary tree split at node k
+ *                of the i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param threshold threshold[i][k] stores the threshold used in the binary tree split at node k of
+ *                  the i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param value value[i][k] stores the leaf output of node k of the i-th tree. This is only defined
+ *              if node k is a leaf node.
+ * \param n_node_samples n_node_samples[i][k] stores the number of data samples associated with
+ *                       node k of the i-th tree.
+ * \param impurity impurity[i][k] stores the impurity measure (gini, entropy etc) associated with
+ *                 node k of the i-th tree.
+ * \param out pointer to store the loaded model
+ * \return 0 for success, -1 for failure
+ */
+TREELITE_DLL int TreeliteLoadSKLearnRandomForestClassifier(
+    int n_estimators, int n_features, int n_classes, const int64_t* node_count,
+    const int64_t** children_left, const int64_t** children_right, const int64_t** feature,
+    const double** threshold, const double** value, const int64_t** n_node_samples,
+    const double** impurity, ModelHandle* out);
+
+/*!
+ * \brief Load a scikit-learn gradient boosting regressor model from a collection of arrays. Refer
+ *        to https://scikit-learn.org/stable/auto_examples/tree/plot_unveil_tree_structure.html to
+ *        learn the mearning of the arrays in detail.
+ * \param n_estimators number of trees in the random forest
+ * \param n_features number of features in the training data
+ * \param node_count node_count[i] stores the number of nodes in the i-th tree
+ * \param children_left children_left[i][k] stores the ID of the left child node of node k of the
+ *                      i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param children_right children_right[i][k] stores the ID of the right child node of node k of the
+ *                       i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param feature feature[i][k] stores the ID of the feature used in the binary tree split at node k
+ *                of the i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param threshold threshold[i][k] stores the threshold used in the binary tree split at node k of
+ *                  the i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param value value[i][k] stores the leaf output of node k of the i-th tree. This is only defined
+ *              if node k is a leaf node.
+ * \param n_node_samples n_node_samples[i][k] stores the number of data samples associated with
+ *                       node k of the i-th tree.
+ * \param impurity impurity[i][k] stores the impurity measure (gini, entropy etc) associated with
+ *                 node k of the i-th tree.
+ * \param out pointer to store the loaded model
+ * \return 0 for success, -1 for failure
+ */
+TREELITE_DLL int TreeliteLoadSKLearnGradientBoostingRegressor(
+    int n_estimators, int n_features, const int64_t* node_count, const int64_t** children_left,
+    const int64_t** children_right, const int64_t** feature, const double** threshold,
+    const double** value, const int64_t** n_node_samples, const double** impurity,
+    ModelHandle* out);
+
+/*!
+ * \brief Load a scikit-learn gradient boosting classifier model from a collection of arrays. Refer
+ *        to https://scikit-learn.org/stable/auto_examples/tree/plot_unveil_tree_structure.html to
+ *        learn the mearning of the arrays in detail.
+ * \param n_estimators number of trees in the random forest
+ * \param n_features number of features in the training data
+ * \param n_classes number of classes in the target variable
+ * \param node_count node_count[i] stores the number of nodes in the i-th tree
+ * \param children_left children_left[i][k] stores the ID of the left child node of node k of the
+ *                      i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param children_right children_right[i][k] stores the ID of the right child node of node k of the
+ *                       i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param feature feature[i][k] stores the ID of the feature used in the binary tree split at node k
+ *                of the i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param threshold threshold[i][k] stores the threshold used in the binary tree split at node k of
+ *                  the i-th tree. This is only defined if node k is an internal (non-leaf) node.
+ * \param value value[i][k] stores the leaf output of node k of the i-th tree. This is only defined
+ *              if node k is a leaf node.
+ * \param n_node_samples n_node_samples[i][k] stores the number of data samples associated with
+ *                       node k of the i-th tree.
+ * \param impurity impurity[i][k] stores the impurity measure (gini, entropy etc) associated with
+ *                 node k of the i-th tree.
+ * \param out pointer to store the loaded model
+ * \return 0 for success, -1 for failure
+ */
+TREELITE_DLL int TreeliteLoadSKLearnGradientBoostingClassifier(
+    int n_estimators, int n_features, int n_classes, const int64_t* node_count,
+    const int64_t** children_left, const int64_t** children_right, const int64_t** feature,
+    const double** threshold, const double** value, const int64_t** n_node_samples,
+    const double** impurity, ModelHandle* out);
+
 /*!
  * \brief Query the number of trees in the model
  * \param handle model to query
@@ -202,6 +327,24 @@ TREELITE_DLL int TreeliteQueryNumClass(ModelHandle handle, size_t* out);
  */
 TREELITE_DLL int TreeliteSetTreeLimit(ModelHandle handle, size_t limit);
 
+/*!
+ * \brief Serialize (persist) a model object to disk
+ * \param filename name of the file to which to serialize the model. The file will be using a
+ *                 binary format that's optimized to store the Treelite model object efficiently.
+ * \param handle handle to the model object
+ * \return 0 for success, -1 for failure
+ */
+TREELITE_DLL int TreeliteSerializeModel(const char* filename, ModelHandle handle);
+
+/*!
+ * \brief Deserialize (load) a model object from disk
+ * \param filename name of the file from which to deserialize the model. The file should be created
+ *                 by a call to TreeliteSerializeModel().
+ * \param handle handle to the model object
+ * \return 0 for success, -1 for failure
+ */
+TREELITE_DLL int TreeliteDeserializeModel(const char* filename, ModelHandle* out);
+
 /*!
  * \brief delete model from memory
  * \param handle model to remove
@@ -210,6 +353,21 @@ TREELITE_DLL int TreeliteSetTreeLimit(ModelHandle handle, size_t limit);
 TREELITE_DLL int TreeliteFreeModel(ModelHandle handle);
 /*! \} */
 
+/*!
+ * \defgroup gtil
+ * General Tree Inference Library (GTIL), providing a reference implementation for
+ * predicting with decision trees. GTIL is useful in cases it is infeasible to build the
+ * tree models as native shared libs.
+ * \{
+ */
+
+TREELITE_DLL int TreeliteGTILGetPredictOutputSize(ModelHandle handle, size_t num_row, size_t* out);
+
+TREELITE_DLL int TreeliteGTILPredict(ModelHandle handle, const float* input, size_t num_row,
+                                     float* output, int pred_transform, size_t* out_result_size);
+
+/*! \} */
+
 /*!
  * \defgroup model_builder
  * Model builder interface: build trees incrementally
