@@ -16,6 +16,7 @@
 #include <treelite/frontend.h>
 #include <treelite/tree.h>
 #include <treelite/math.h>
+#include <treelite/gtil.h>
 #include <dmlc/thread_local.h>
 #include <memory>
 #include <algorithm>
@@ -176,12 +177,100 @@ int TreeliteLoadXGBoostModelFromMemoryBuffer(const void* buf, size_t len, ModelH
   API_END();
 }
 
+int TreeliteLoadSKLearnRandomForestRegressor(
+    int n_estimators, int n_features, const int64_t* node_count, const int64_t** children_left,
+    const int64_t** children_right, const int64_t** feature, const double** threshold,
+    const double** value, const int64_t** n_node_samples, const double** impurity,
+    ModelHandle* out) {
+  API_BEGIN();
+  std::unique_ptr<Model> model = frontend::LoadSKLearnRandomForestRegressor(
+      n_estimators, n_features, node_count, children_left, children_right, feature, threshold,
+      value, n_node_samples, impurity);
+  *out = static_cast<ModelHandle>(model.release());
+  API_END();
+}
+
+int TreeliteLoadSKLearnRandomForestClassifier(
+    int n_estimators, int n_features, int n_classes, const int64_t* node_count,
+    const int64_t** children_left, const int64_t** children_right, const int64_t** feature,
+    const double** threshold, const double** value, const int64_t** n_node_samples,
+    const double** impurity, ModelHandle* out) {
+  API_BEGIN();
+  std::unique_ptr<Model> model = frontend::LoadSKLearnRandomForestClassifier(
+      n_estimators, n_features, n_classes, node_count, children_left, children_right, feature,
+      threshold, value, n_node_samples, impurity);
+  *out = static_cast<ModelHandle>(model.release());
+  API_END();
+}
+
+int TreeliteLoadSKLearnGradientBoostingRegressor(
+    int n_estimators, int n_features, const int64_t* node_count, const int64_t** children_left,
+    const int64_t** children_right, const int64_t** feature, const double** threshold,
+    const double** value, const int64_t** n_node_samples, const double** impurity,
+    ModelHandle* out) {
+  API_BEGIN();
+  std::unique_ptr<Model> model = frontend::LoadSKLearnGradientBoostingRegressor(
+      n_estimators, n_features, node_count, children_left, children_right, feature, threshold,
+      value, n_node_samples, impurity);
+  *out = static_cast<ModelHandle>(model.release());
+  API_END();
+}
+
+int TreeliteLoadSKLearnGradientBoostingClassifier(
+    int n_estimators, int n_features, int n_classes, const int64_t* node_count,
+    const int64_t** children_left, const int64_t** children_right, const int64_t** feature,
+    const double** threshold, const double** value, const int64_t** n_node_samples,
+    const double** impurity, ModelHandle* out) {
+  API_BEGIN();
+  std::unique_ptr<Model> model = frontend::LoadSKLearnGradientBoostingClassifier(
+      n_estimators, n_features, n_classes, node_count, children_left, children_right, feature,
+      threshold, value, n_node_samples, impurity);
+  *out = static_cast<ModelHandle>(model.release());
+  API_END();
+}
+
+int TreeliteSerializeModel(const char* filename, ModelHandle handle) {
+  API_BEGIN();
+  FILE* fp = std::fopen(filename, "wb");
+  CHECK(fp) << "Failed to open file '" << filename << "'";
+  auto* model_ = static_cast<Model*>(handle);
+  model_->SerializeToFile(fp);
+  std::fclose(fp);
+  API_END();
+}
+
+int TreeliteDeserializeModel(const char* filename, ModelHandle* out) {
+  API_BEGIN();
+  FILE* fp = std::fopen(filename, "rb");
+  CHECK(fp) << "Failed to open file '" << filename << "'";
+  std::unique_ptr<Model> model = Model::DeserializeFromFile(fp);
+  std::fclose(fp);
+  *out = static_cast<ModelHandle>(model.release());
+  API_END();
+}
+
 int TreeliteFreeModel(ModelHandle handle) {
   API_BEGIN();
   delete static_cast<Model*>(handle);
   API_END();
 }
 
+int TreeliteGTILGetPredictOutputSize(ModelHandle handle, size_t num_row, size_t* out) {
+  API_BEGIN();
+  const auto* model_ = static_cast<const Model*>(handle);
+  *out = gtil::GetPredictOutputSize(model_, num_row);
+  API_END();
+}
+
+int TreeliteGTILPredict(ModelHandle handle, const float* input, size_t num_row, float* output,
+                        int pred_transform, size_t* out_result_size) {
+  API_BEGIN();
+  const auto* model_ = static_cast<const Model*>(handle);
+  *out_result_size =
+      gtil::Predict(model_, input, num_row, output, (pred_transform == 1));
+  API_END();
+}
+
 int TreeliteQueryNumTree(ModelHandle handle, size_t* out) {
   API_BEGIN();
   const auto* model_ = static_cast<const Model*>(handle);
