@@ -5,6 +5,7 @@
  * \author Hyunsu Cho
  */
 
+#include "xgboost/xgboost.h"
 #include <dmlc/data.h>
 #include <dmlc/memory_io.h>
 #include <treelite/frontend.h>
@@ -16,7 +17,7 @@
 
 namespace {
 
-treelite::Model ParseStream(dmlc::Stream* fi);
+inline std::unique_ptr<treelite::Model> ParseStream(dmlc::Stream* fi);
 
 }  // anonymous namespace
 
@@ -25,14 +26,14 @@ namespace frontend {
 
 DMLC_REGISTRY_FILE_TAG(xgboost);
 
-void LoadXGBoostModel(const char* filename, Model* out) {
+std::unique_ptr<treelite::Model> LoadXGBoostModel(const char* filename) {
   std::unique_ptr<dmlc::Stream> fi(dmlc::Stream::Create(filename, "r"));
-  *out = std::move(ParseStream(fi.get()));
+  return ParseStream(fi.get());
 }
 
-void LoadXGBoostModel(const void* buf, size_t len, Model* out) {
+std::unique_ptr<treelite::Model> LoadXGBoostModel(const void* buf, size_t len) {
   dmlc::MemoryFixedSizeStream fs(const_cast<void*>(buf), len);
-  *out = std::move(ParseStream(&fs));
+  return ParseStream(&fs);
 }
 
 }  // namespace frontend
@@ -43,15 +44,6 @@ namespace {
 
 typedef float bst_float;
 
-struct ProbToMargin {
-  static float Sigmoid(float global_bias) {
-    return -logf(1.0f / global_bias - 1.0f);
-  }
-  static float Exponential(float global_bias) {
-    return logf(global_bias);
-  }
-};
-
 /* peekable input stream implemented with a ring buffer */
 class PeekableInputStream {
  public:
@@ -330,7 +322,7 @@ class XGBTree {
   }
 };
 
-inline treelite::Model ParseStream(dmlc::Stream* fi) {
+inline std::unique_ptr<treelite::Model> ParseStream(dmlc::Stream* fi) {
   std::vector<XGBTree> xgb_trees_;
   LearnerModelParam mparam_;    // model parameter
   GBTreeModelParam gbm_param_;  // GBTree training parameter
@@ -375,9 +367,9 @@ inline treelite::Model ParseStream(dmlc::Stream* fi) {
   }
 
   /* loading GBTree */
-  CHECK_EQ(name_gbm_, "gbtree")
+  CHECK(name_gbm_ == "gbtree" || name_gbm_ == "dart")
     << "Invalid XGBoost model file: "
-    << "Gradient booster must be gbtree type.";
+    << "Gradient booster must be gbtree or dart type.";
 
   CHECK_EQ(fp->Read(&gbm_param_, sizeof(gbm_param_)), sizeof(gbm_param_))
     << "Invalid XGBoost model file: corrupted GBTree parameters";
@@ -386,50 +378,57 @@ inline treelite::Model ParseStream(dmlc::Stream* fi) {
     xgb_trees_.back().Load(fp.get());
   }
   CHECK_EQ(gbm_param_.num_roots, 1) << "multi-root trees not supported";
-
-  // Before XGBoost 1.0.0, the global bias saved in model is a transformed value.  After
-  // 1.0 it's the original value provided by user.
-  bool need_transform_to_margin = mparam_.major_version >= 1;
-
-  /* 2. Export model */
-  treelite::Model model;
-  model.num_feature = mparam_.num_feature;
-  model.num_output_group = std::max(mparam_.num_class, 1);
-  model.random_forest_flag = false;
-
-  // set global bias
-  model.param.global_bias = static_cast<float>(mparam_.base_score);
-  std::vector<std::string> exponential_family {
-    "count:poisson", "reg:gamma", "reg:tweedie"
-  };
-  if (need_transform_to_margin) {
-    if (name_obj_ == "reg:logistic" || name_obj_ == "binary:logistic") {
-      model.param.global_bias = ProbToMargin::Sigmoid(model.param.global_bias);
-    } else if (std::find(exponential_family.cbegin() , exponential_family.cend(), name_obj_)
-               != exponential_family.cend()) {
-      model.param.global_bias = ProbToMargin::Exponential(model.param.global_bias);
+  // tree_info is currently unused.
+  std::vector<int> tree_info;
+  tree_info.resize(gbm_param_.num_trees);
+  if (gbm_param_.num_trees != 0) {
+    CHECK_EQ(fp->Read(dmlc::BeginPtr(tree_info), sizeof(int32_t) * tree_info.size()),
+             sizeof(int32_t) * tree_info.size());
+  }
+  // Load weight drop values (per tree) for dart models.
+  std::vector<bst_float> weight_drop;
+  if (name_gbm_ == "dart") {
+    weight_drop.resize(gbm_param_.num_trees);
+    if (gbm_param_.num_trees != 0) {
+      fi->Read(&weight_drop);
     }
   }
 
-  // set correct prediction transform function, depending on objective function
-  if (name_obj_ == "multi:softmax") {
-    std::strncpy(model.param.pred_transform, "max_index", sizeof(model.param.pred_transform));
-  } else if (name_obj_ == "multi:softprob") {
-    std::strncpy(model.param.pred_transform, "softmax", sizeof(model.param.pred_transform));
-  } else if (name_obj_ == "reg:logistic" || name_obj_ == "binary:logistic") {
-    std::strncpy(model.param.pred_transform, "sigmoid", sizeof(model.param.pred_transform));
-    model.param.sigmoid_alpha = 1.0f;
-  } else if (std::find(exponential_family.cbegin() , exponential_family.cend(), name_obj_)
-             != exponential_family.cend()) {
-    std::strncpy(model.param.pred_transform, "exponential", sizeof(model.param.pred_transform));
+  /* 2. Export model */
+  std::unique_ptr<treelite::Model> model_ptr = treelite::Model::Create<float, float>();
+  auto* model = dynamic_cast<treelite::ModelImpl<float, float>*>(model_ptr.get());
+  model->num_feature = static_cast<int>(mparam_.num_feature);
+  model->average_tree_output = false;
+  const int num_class = std::max(mparam_.num_class, 1);
+  if (num_class > 1) {
+    // multi-class classifier
+    model->task_type = treelite::TaskType::kMultiClfGrovePerClass;
+    model->task_param.grove_per_class = true;
   } else {
-    std::strncpy(model.param.pred_transform, "identity", sizeof(model.param.pred_transform));
+    // binary classifier or regressor
+    model->task_type = treelite::TaskType::kBinaryClfRegr;
+    model->task_param.grove_per_class = false;
+  }
+  model->task_param.output_type = treelite::TaskParameter::OutputType::kFloat;
+  model->task_param.num_class = num_class;
+  model->task_param.leaf_vector_size = 1;
+
+  // set correct prediction transform function, depending on objective function
+  treelite::details::xgboost::SetPredTransform(name_obj_, &model->param);
+
+  // set global bias
+  model->param.global_bias = static_cast<float>(mparam_.base_score);
+  // Before XGBoost 1.0.0, the global bias saved in model is a transformed value.  After
+  // 1.0 it's the original value provided by user.
+  const bool need_transform_to_margin = mparam_.major_version >= 1;
+  if (need_transform_to_margin) {
+    treelite::details::xgboost::TransformGlobalBiasToMargin(&model->param);
   }
 
   // traverse trees
   for (const auto& xgb_tree : xgb_trees_) {
-    model.trees.emplace_back();
-    treelite::Tree& tree = model.trees.back();
+    model->trees.emplace_back();
+    treelite::Tree<float, float>& tree = model->trees.back();
     tree.Init();
 
     // assign node ID's so that a breadth-wise traversal would yield
@@ -443,14 +442,17 @@ inline treelite::Model ParseStream(dmlc::Stream* fi) {
       const XGBTree::Node& node = xgb_tree[old_id];
       const NodeStat stat = xgb_tree.Stat(old_id);
       if (node.is_leaf()) {
-        const bst_float leaf_value = node.leaf_value();
-        tree.SetLeaf(new_id, static_cast<treelite::tl_float>(leaf_value));
+        bst_float leaf_value = node.leaf_value();
+        // Fold weight drop into leaf value for dart models.
+        if (!weight_drop.empty()) {
+          leaf_value *= weight_drop[model->trees.size() - 1];
+        }
+        tree.SetLeaf(new_id, static_cast<float>(leaf_value));
       } else {
         const bst_float split_cond = node.split_cond();
         tree.AddChilds(new_id);
         tree.SetNumericalSplit(new_id, node.split_index(),
-            static_cast<treelite::tl_float>(split_cond), node.default_left(),
-            treelite::Operator::kLT);
+            static_cast<float>(split_cond), node.default_left(), treelite::Operator::kLT);
         tree.SetGain(new_id, stat.loss_chg);
         Q.push({node.cleft(), tree.LeftChild(new_id)});
         Q.push({node.cright(), tree.RightChild(new_id)});
@@ -458,7 +460,7 @@ inline treelite::Model ParseStream(dmlc::Stream* fi) {
       tree.SetSumHess(new_id, stat.sum_hess);
     }
   }
-  return model;
+  return model_ptr;
 }
 
 }  // anonymous namespace
